@@ -81,6 +81,15 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 ### either a NULL or an integer vector of valid indices along the
 ### corresponding dimension in 'x'.
 
+.Nindex2Noffs <- function(Nindex)
+{
+    stopifnot(is.list(Nindex))
+    lapply(Nindex,
+        function(subscript)
+            if (is.null(subscript)) NULL else subscript - 1L
+    )
+}
+
 ### 'Rvector' is considered "short" if it can be cleanly recycled along the
 ### first (a.k.a. leftmost or innermost) dimension of the array selection.
 ### This is a requirement of .subassign_SVT_with_short_Rvector().
@@ -90,10 +99,9 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 ### array selection (in other words it cannot contain zeros).
 ### Returns TRUE or FALSE indicating whether 'Rvector' is considered "short"
 ### or not.
-.is_short <- function(Rvector, selection_dim)
+.is_short <- function(Rvector_len, selection_dim)
 {
-    stopifnot(is.vector(Rvector), is.integer(selection_dim))
-    Rvector_len <- length(Rvector)
+    stopifnot(isSingleInteger(Rvector_len), is.integer(selection_dim))
     if (Rvector_len == 0L)
         stop(wmsg("right value has length zero"))
     selection_dim[[1L]] %% Rvector_len == 0L
@@ -124,21 +132,35 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
     if (any(selection_dim == 0L))
         return(x)
 
-    stopifnot(.is_short(Rvector, selection_dim))
+    Rvector_len <- length(Rvector)
+    stopifnot(.is_short(Rvector_len, selection_dim))
+
+    ## Prepare 'Noffs' and 'Rvector'.
+    Norder <- S4Arrays:::get_Nindex_order(Nindex)
+    Nindex <- S4Arrays:::subset_Nindex_by_Nindex(Nindex, Norder)
+    Noffs <- .Nindex2Noffs(Nindex)
+    Norder1 <- Norder[[1L]]
+    if (!is.null(Norder1))
+        Rvector <- Rvector[((Norder1 - 1L) %% Rvector_len) + 1L]
+    storage.mode(Rvector) <- new_type
 
-    storage.mode(Rvector) <- type(x)
     new_SVT <- SparseArray.Call("C_subassign_SVT_with_short_Rvector",
-                                x@dim, x@type, x@SVT, Nindex, Rvector)
+                                x@dim, x@type, x@SVT, Noffs, Rvector)
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
-.Nindex2Noffs <- function(Nindex)
+.adjust_right_array_dim <- function(right_array, selection_dim)
 {
-    stopifnot(is.list(Nindex))
-    lapply(Nindex,
-        function(subscript)
-            if (is.null(subscript)) NULL else subscript - 1L
-    )
+    right_dim <- unname(dim(right_array))
+    if (identical(selection_dim, right_dim))
+        return(right_array)
+    effdim_idx1 <- which(selection_dim != 1L)
+    effdim_idx2 <- which(right_dim != 1L)
+    if (!identical(selection_dim[effdim_idx1], right_dim[effdim_idx2]))
+        stop(wmsg("dimensions of right array don't ",
+                  "match dimensions of array selection"))
+    dim(right_array) <- selection_dim
+    right_array
 }
 
 .subassign_SVT_with_Rarray <- function(x, Nindex, Rarray)
@@ -153,9 +175,7 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 
     ## No-op (except for type change above) if array selection is empty.
     selection_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
-    if (!identical(selection_dim, unname(dim(Rarray))))
-        stop(wmsg("dimensions of right array don't ",
-                  "match dimensions of array selection"))
+    Rarray <- .adjust_right_array_dim(Rarray, selection_dim)
     if (any(selection_dim == 0L))
         return(x)
 
@@ -171,35 +191,33 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
-.subassign_SVT_with_SVT <- function(x, Nindex, v)
+.subassign_SVT_with_SVT <- function(x, Nindex, y)
 {
     stopifnot(is(x, "SVT_SparseArray"), is.list(Nindex))
     check_svt_version(x)
-    stopifnot(is(v, "SVT_SparseArray"))
-    check_svt_version(v)
+    stopifnot(is(y, "SVT_SparseArray"))
+    check_svt_version(y)
 
     ## Change 'x' type if necessary.
-    new_type <- type(c(vector(type(x)), vector(type(v))))
+    new_type <- type(c(vector(type(x)), vector(type(y))))
     type(x) <- new_type
 
     ## No-op (except for type change above) if array selection is empty.
     selection_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
-    if (!identical(selection_dim, unname(dim(v))))
-        stop(wmsg("dimensions of right array don't ",
-                  "match dimensions of array selection"))
+    y <- .adjust_right_array_dim(y, selection_dim)
     if (any(selection_dim == 0L))
         return(x)
 
-    ## Prepare 'Noffs' and 'v'.
+    ## Prepare 'Noffs' and 'y'.
     Norder <- S4Arrays:::get_Nindex_order(Nindex)
     Nindex <- S4Arrays:::subset_Nindex_by_Nindex(Nindex, Norder)
     Noffs <- .Nindex2Noffs(Nindex)
-    v <- S4Arrays:::subset_by_Nindex(v, Norder)
-    type(v) <- new_type
+    y <- S4Arrays:::subset_by_Nindex(y, Norder)
+    type(y) <- new_type
 
     new_SVT <- SparseArray.Call("C_subassign_SVT_with_SVT",
-                                x@dim, x@type, x@SVT, Noffs,
-                                v@dim, v@type, v@SVT)
+                                x@dim, x@type, x@SVT, FALSE, Noffs,
+                                y@dim, y@type, y@SVT, FALSE)
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
@@ -238,7 +256,7 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
         if (any(selection_dim == 0L))
             return(x)
 
-        if (.is_short(value, selection_dim))
+        if (.is_short(length(value), selection_dim))
             return(.subassign_SVT_with_short_Rvector(x, Nindex, value))
 
         ## Turn 'value' into an ordinary array of same dimensions as
